import posixpath,os
SRC = "_source"
def srcpath(p:str)->str:
    return posixpath.join(SRC,p)
ROOT_SPECFILE_PATH = srcpath("webcomic.yaml")
ASSETS = "_assets"
def assetpath(p:str)->os.path:
    return posixpath.join(ASSETS,p)
STATIC = "static"
def staticpath(p:str)->os.path:
    return posixpath.join(STATIC,p)

DEBUG = ".debug"
def debugpath(p:str)->os.path:
    return posixpath.join(DEBUG,p)


from . import wctree
import click
import os
from distutils.dir_util import copy_tree
import pathlib
import json
from . import localpaths
import errno, os, stat, shutil

def handleRemoveReadonly(func, path, exc):
  excvalue = exc[1]
  if func in (os.rmdir, os.remove) and excvalue.errno == errno.EACCES:
      os.chmod(path, stat.S_IRWXU| stat.S_IRWXG| stat.S_IRWXO) # 0777
      func(path)
  else:
      raise

def rmdir(directory):
    directory = pathlib.Path(directory)
    for item in directory.iterdir():
        if item.is_dir():
            rmdir(item)
        else:
            item.unlink()
    directory.rmdir()



def build_website():
    wc = wctree.WCTree()
    wc.build()


def create_website():
    if len(os.listdir(os.getcwd()))>0:
        print(os.listdir(os.getcwd()))
        click.echo("The current folder is not empty! Cannot setup template website.",err=True)
        return False
    
    click.echo("Assembling template webcomic...")
    from_dir = os.path.join(os.path.dirname(__file__),'../sample_comic')
    to_dir = os.getcwd()
    copy_tree(from_dir,to_dir)

    pathlib.Path('_assets').mkdir(exist_ok=True)

    click.echo("Done!")
    return


def clean_website():
    if pathlib.Path('_source/webcomic.yaml').is_file():
        # for item in os.listdir(os.getcwd()):
        #     if not item.startswith("_"):
        #         print(f"Delete {item}")
        try:
            dalist = json.load(open(localpaths.debugpath('buildlist.json'),'r'))
        except FileNotFoundError:
            click.echo("No buildlist.json file found... looks like you'll have to clean by hand.")
            return
        for item in dalist:
            print(f"Delete {item}")
            path = pathlib.Path(item)
            if path.is_dir():
                shutil.rmtree(item,ignore_errors=False,onerror=handleRemoveReadonly)
            elif path.is_file():
                os.remove(path)
            else:
                pass
            # if path.is_dir():
            #     rmdir(path)
            # elif path.is_file():
            #     path.unlink()
            # else:
            #     click.echo("???")

        click.echo("Done!")
    else:
        click.echo("Can't find webcomic.yaml! Is this even the right folder?",err=True)


import markdown
from functools import cached_property
from markdown import Markdown
from io import StringIO

# from https://stackoverflow.com/questions/761824/python-how-to-convert-markdown-formatted-text-to-text
def unmark_element(element, stream=None):
    if stream is None:
        stream = StringIO()
    if element.text:
        stream.write(element.text)
    for sub in element:
        unmark_element(sub, stream)
    if element.tail:
        stream.write(element.tail)
    return stream.getvalue()

Markdown.output_formats["plain"] = unmark_element
plainmd = Markdown(output_format="plain")
plainmd.stripTopLevelTags = False


class Blob:
    def __init__(self,source:str):
        self.source = source

    @cached_property
    def html(self)->str:
        rdr = markdown.markdown(self.source)
        return rdr

    @cached_property
    def plaintext(self)->str:
        return plainmd.convert(self.source)

    def __bool__(self):
        return self.source != ''


from .wcnode import WCNode
from feedgen.feed import FeedGenerator
import pytz
from functools import cached_property

class Feed:
    def __init__(self, entries: list[WCNode], webroot:str, metadata:dict) -> None:
        self.entries = entries
        self.webroot = webroot
        self.metadata = metadata

    @cached_property
    def generator(self)->FeedGenerator:
        fg = FeedGenerator()
        fg.title(self.metadata['title'])
        fg.description(self.metadata['description'])
        fg.link(dict(href=self.webroot,rel='alternate'))
        fg.id(self.webroot)

        for entry in self.entries:
            fe = fg.add_entry()
            permalink = entry.permalink(self.webroot)
            fe.id(permalink)
            fe.title(entry.full_title)
            fe.link(dict(href=permalink,rel='alternate'))

            pic = entry.feed_img
            desc = f"<img src='{pic.absolute_url(self.webroot)}' width='{pic.image_width}' height='{pic.image_height}' alt='{entry.full_title}'></hr>{entry.description.html}"
            fe.description(desc)
            fe.published(pytz.utc.localize(entry.date))
        
        
        return fg

    @cached_property
    def rss(self)->str:
        return self.generator.rss_str(pretty=True).decode('utf-8')
    @cached_property
    def atom(self)->str:
        return self.generator.atom_str(pretty=True).decode('utf-8')

    def build(self):
        with open('rss.xml','w') as rss_f:
            rss_f.write(self.rss)
        with open('atom.xml','w') as atom_f:
            atom_f.write(self.atom)


import click
import sys,os
sys.path.append(os.path.dirname(__file__))

import builder
import webserver

from __init__ import __version__


@click.group('guttacli')
@click.pass_context
def cli(ctx):
    pass

@cli.command('version')
def command_version():
    click.echo(__version__)

@cli.command("create")
def command_create():
    builder.create_website()

@cli.command("build")
def command_build():
    click.echo("Building your webcomic...")
    builder.build_website()
    click.echo("Done!")


@cli.command("go")
def command_go():
    click.echo("Building your webcomic...")
    builder.build_website()
    click.echo("Starting webserver at "+click.style("http://127.0.0.1:8069/",fg='red')+" , go there in your browser...")
    webserver.run_webserver()
    click.echo("Server closed.")


@cli.command('clean')
def command_clean():
    builder.clean_website()
    

def main():
    """A static website builder for webcomics!"""
    cli(prog_name="gutta")

if __name__ == '__main__':
    main()




import oyaml as yaml
from .exceptions import MissingOption,YAMLParsingError,MissingSpecFile


def read_yaml(path):
    try:
        with open(path,'r') as stream:
            try:
                return yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print(exc)
                raise YAMLParsingError(exc)
    except FileNotFoundError:
        raise MissingSpecFile(path)



def getopt(config:dict, key:str, default = None):
    if key in config:
        return config[key]
    else:
        if default != None:
            return default
        else:
            raise MissingOption(f"Required option '{key}' is missing.")


import oyaml as yaml

class BuildError(Exception):
    pass
class MissingSpecFile(BuildError):
    def __init__(self,path) -> None:
        super().__init__(f"Required source webcomic file {path} not found. Are you sure you're in the right folder?")


class ParsingError(BuildError):
    pass
class YAMLParsingError(ParsingError):
    def __init__(self, yaml_err : yaml.YAMLError) -> None:
        super().__init__(f"Something wrong with this yaml file...")
class MissingOption(ParsingError):
    pass

class NotIdentifier(ParsingError):
    pass
class UnknownHierarchyLevel(ParsingError):
    def __init__(self, lvl_name:str) -> None:
        super().__init__(f"The hierarchy level '{lvl_name}' is referenced, but was not defined in the hierarchy. Maybe it's a typo?")



class TemplateNotFound(BuildError):
    pass


class EmptyTrickle(BuildError):
    def __init__(self,nid:str,title:str):
        super().__init__(f"Node {nid} ({title}) is marked as trickle but it has no children.")

class BadPath(ParsingError):
    def __init__(self,path:str) -> None:
        super().__init__(f"The following referenced path is malformed: {path}")


class DatingError(BuildError):
    pass


class NoDates(DatingError):
    def __init__(self) -> None:
        super().__init__("There are leaf nodes marked as 'dated' but none of them have a date, so I cannot interpolate.")
class DateParsingFailure(DatingError):
    def __init__(self, datestr:str) -> None:
        super().__init__(f"The following date string could not be parsed as a valid date: '{datestr}'.")
class NoChildDates(DatingError):
    def __init__(self, pnid:str) -> None:
        super().__init__(f"A date was requested to node '{pnid}' which is an undated leaf. This is probably because you've toggled showing dates for parent nodes without dated children.")


class UnsupportedFeature(BuildError):
    def __init__(self, feat:str) -> None:
        super().__init__(f"Unsupported: {feat}")


import os
import shutil

def respath(fname:str)->str:
    return os.path.join(os.path.dirname(__file__),"resources",fname)

def read_resource(fname:str)->str:
    path = respath(fname)
    return open(path,'r').read()

def move_resource(fname:str,dest:str):
    path = respath(fname)
    shutil.copyfile(path,os.path.join(dest,fname))